"""
This module defines actions, or effects which can be performed on SiGL.graph.Node objects.
It requires the use of NanoThreads, so that each effect is rendered asynchronously.
"""

import nanothreads
import time
from vector import Vector
from math import log,pi,sin,cos
import random

def time_range(start, stop, duration):
    diff = stop - start
    stop_time = time.clock() + duration
    now = time.clock()
    while now < stop_time:
        time_left = stop_time - now
        yield stop - time_left/duration * diff
        now = time.clock()
    yield stop
        
def sine_range(start, stop, duration):
    diff = stop - start
    stop_time = time.clock() + duration
    now = time.clock()
    HALFPI = (pi/2.0)
    while now < stop_time:
        time_left = stop_time - now
        ratio = time_left/duration
        pos = ratio * pi
        offset = (sin(pos) * diff)/2.0
        if ratio >= 0.5:
            yield start + offset
        else:
            yield stop - offset
        now = time.clock()
    yield stop
        

def wait(duration):
    stop = time.clock() + duration
    while time.clock() < stop: yield None


class Action(object):
    def __init__(self, node, **kw):
        self.node = node
        self.remove=False
        if 'remove_when_done' in kw:
            self.remove = kw['remove_when_done']
            del kw['remove_when_done']
        self.kw = kw
    def execute(self, *args, **kw):
        """
        Execute an action on self.node. Checks that the action isn't already running on self.node.
        """
        #print "Started action:", self.__class__.__name__
        if hasattr(self.node, self.__class__.__name__): return
        setattr(self.node, self.__class__.__name__, self)
        thread = nanothreads.register(self._act, *args, **self.kw)
        nanothreads.register_exit(thread, self.cleanup)
        return thread

    def cleanup(self):
        """
        Called when action is finished, removes node from group. Overide to change behavior.
        """
        #print "Finished action:", self.__class__.__name__
        if self.remove:
            try:
                self.node.container.remove(self.node)
            except AttributeError:
                pass

    def _act(self, *args, **kw):
        #print '_act_', self.__class__.__name__
        for i in self.act(*args, **kw): yield i
        delattr(self.node, self.__class__.__name__)


    def act(self, *args, **kw):
        """
        Override this generator method to implement behavior.
        """
        yield None


class Nothing(Action):
    """
    Do Nothing. A Null Action.
    """
    def act(self):
        yield None


class Move(Action):
    """
    Move from current position to a new position over duration seconds.
    """
    def act(self, duration=1.0, position=[0.0,0.0], soft=False,callback=None):
        start = Vector(self.node.position)
        stop = Vector(position)
        diff = stop - start
        if soft:
            rng = sine_range(0,1,duration)
        else:
            rng = time_range(0,1,duration)
        for i in rng:
            self.node.position = start + (diff*i)
            if callback: callback(self.node.position)
            yield None
        self.node.position = stop

class Explode(Action):
    """
    Scales up a node to node.scale * size over duration seconds.
    """
    def act(self, duration=1.0, size=2.0):
        o_scale = self.node.scale * size
        s_scale = self.node.scale
        for i in time_range(0,1,duration):
            self.node.scale =   s_scale + (o_scale * i)
            yield None


class FadeOut(Action):
    """
    Decreases node opacity over duration seconds.
    """
    def act(self, duration=1.0):
        s_alpha = self.node.color[3]
        for i in time_range(0,1,duration):
            self.node.color[3] =   s_alpha - i
            yield None

class FadeIn(Action):
    """
    Increases node opacity over duration seconds.
    """
    def act(self, duration=1.0,amount=1.0):
        s_alpha = self.node.color[3]
        for i in time_range(0,amount,duration):
            self.node.color[3] =  i
            yield None

class Implode(Action):
    """
    Reduce node scale to 0 over duration seconds.
    """
    def act(self, duration=1.0, amount=1.0):
        o_scale = self.node.scale
        for i in time_range(0,amount,duration):
            self.node.scale = o_scale - i
            yield None

class Spin(Action):
    """
    Spin a node to degrees in direction.
    """
    def act(self, duration=1.0, direction=1, degrees=360):
        start = float(self.node.rotation)
        for i in time_range(0,degrees,duration):
            self.node.rotation = (start + i) * direction
            yield None
        #self.node.rotation = degrees
        #self.node.rotation = start

class Press(Action):
    def act(self):
        o_scale = self.node.scale
        self.node.scale *= 0.95
        for i in wait(0.1): yield None
        self.node.scale = o_scale

class Pulse(Action):
    """
    Scale up then down amount over duration.
    """
    def act(self, amount=0.01, duration=0.5):
        o_scale = self.node.scale
        for i in time_range(0,amount,duration/2.0):
            self.node.scale = o_scale + i
            yield None
        for i in time_range(0,amount,duration/2.0):
            self.node.scale = o_scale + (amount - i)
            yield None
        self.node.scale = o_scale

class Initialize(Action):
    """
    Translate to position.
    """
    def act(self, **kw):
        for key, value in kw.items():
            setattr(self.node, key, value)
        yield None


class Quake(Action):
    def act(self, amount=0.25, delay=0.1, duration=1.0):
        o_pos = Vector(self.node.position)
        for i in time_range(0, 1.0, duration):
            x = random.uniform(-amount, amount)
            y = random.uniform(-amount, amount)
            self.node.position = o_pos + (x,y)
            for x in wait(delay): yield None
            yield None

        self.node.position = o_pos
class Shake(Action):
    """
    Shake a node amount in each direction, with delay seconds between each frame over duration seconds.
    """
    def act(self, amount=10, delay=0.1, duration=1.0):
        rotation = self.node.rotation

        for i in time_range(0, 1.0, duration/2.0):
            self.node.rotation = rotation
            for x in wait(delay): yield None
            self.node.rotation = rotation + (amount * i)
            for x in wait(delay): yield None
            self.node.rotation = rotation
            for x in wait(delay): yield None
            self.node.rotation = rotation - (amount * i)
            for x in wait(delay): yield None

        for i in time_range(0, 1.0, duration/2.0):
            self.node.rotation = rotation
            for x in wait(delay): yield None
            self.node.rotation = rotation  + (amount - (amount * i))
            for x in wait(delay): yield None
            self.node.rotation = rotation
            for x in wait(delay): yield None
            self.node.rotation = rotation - (amount - (amount * i))
            for x in wait(delay): yield None

        self.node.rotation = rotation


