"""
This module defines the objects (other than SiGL.graph.Group) which can be blitted using screen.blit.
"""

from OpenGL.GL import *
from OpenGL.GLU import *
import display
import vector
import OBJFile
import copy
from base import SiGLObject
import time


class Shape(SiGLObject):
    def __init__(self, **kw):
        self.color=[1.0,1.0,1.0,1.0]
        self.rotation=0.0
        self.scale=1.0
        self.centre_point=(0,0)
        self.screen=display._screen
        self.compile=True
        SiGLObject.__init__(self, **kw)
    
    def copy(self):
        return copy.copy(self)

    def render(self):
        x,y = self.centre_point
        glPushMatrix()
        glTranslatef(x,y,0)
        if self.compile:
            try:
                glCallList(self._dlist)
            except AttributeError:
                self._dlist = glGenLists(1)
                glNewList(self._dlist, GL_COMPILE)
                self.draw()
                glEndList()
                glCallList(self._dlist)
        else:
            self.draw()

        glPopMatrix()

    def draw(self):
        pass

    def __del__(self):
        try:
            glDeleteLists(self._dlist, 1)
        except NameError:
            pass
        except AttributeError:
            pass

class Rectangle(Shape):
    def __init__(self, points, **kw):
        Shape.__init__(self, **kw)
        self.points = points

    def draw(self):
        glBindTexture(GL_TEXTURE_2D, 0)
        (l,t),(r,b) = self.points
        glBegin(GL_QUADS)
        glVertex2f(l,t)
        glVertex2f(r,t)
        glVertex2f(r,b)
        glVertex2f(l,b)
        glEnd()


class Image(Shape):
    def __init__(self, texture, **kw):
        Shape.__init__(self, **kw)
        self.texture = texture
        self.centre_point = (-self.texture.n_size[0]/2,-self.texture.n_size[1]/2)
        try:
            texture.pack.id
        except AttributeError:
            raise AttributeError, 'texture pack has no id attribute, (call texture,pack.build() first)'
        
    def draw(self):
        l,t = self.texture.texels[0]
        r,b = self.texture.texels[1]
        glBindTexture(GL_TEXTURE_2D, self.texture.pack.id)
        glBegin(GL_QUADS)
        w,h = self.texture.n_size
        glTexCoord2f(l,b);glVertex2f(0, 0)
        glTexCoord2f(l,t);glVertex2f(0, h)
        glTexCoord2f(r,t);glVertex2f(w, h)
        glTexCoord2f(r,b);glVertex2f(w, 0)
        glEnd()

class FrameSet(Shape):
    def __init__(self, textures, **kw):
        self.default = None
        Shape.__init__(self, **kw)
        self.textures = textures
        if self.default:
            self.texture = textures[self.default]
        else:
            self.texture = textures.items()[0][1]
        self.centre_point = (-self.texture.n_size[0]/2,-self.texture.n_size[1]/2)
        self.compile=False
        try:
            self.texture.pack.id
        except AttributeError:
            raise AttributeError, 'texture pack has no id attribute, (call texture,pack.build() first)'
    
    def set(self, key):
        self.texture = self.textures[key]

    def draw(self):
        l,t = self.texture.texels[0]
        r,b = self.texture.texels[1]
        glBindTexture(GL_TEXTURE_2D, self.texture.pack.id)
        glBegin(GL_QUADS)
        w,h = self.texture.n_size
        glTexCoord2f(l,b);glVertex2f(0, 0)
        glTexCoord2f(l,t);glVertex2f(0, h)
        glTexCoord2f(r,t);glVertex2f(w, h)
        glTexCoord2f(r,b);glVertex2f(w, 0)
        glEnd()


class Animation(Shape):
    def __init__(self, textures, **kw):
        self.loop = True
        self.fps = 1
        Shape.__init__(self, **kw)
        self.compile = False
        self.textures = textures
        self.index = 0
        self.texture = self.textures[self.index]
        self.centre_point = (-self.texture.n_size[0]/2,-self.texture.n_size[1]/2)
        self.last_call = time.clock()
        try:
            self.texture.pack.id
        except AttributeError:
            raise AttributeError, 'texture pack has no id attribute, (call texture,pack.build() first)'
            
    def next(self):
        duration = time.clock() - self.last_call
        
        step = 1.0 / self.fps
        if duration < step:
            return
        self.last_call = time.clock()
        self.index += 1
        if self.index >= len(self.textures):
            if self.loop:
                self.index = 0
            else:
                self.index = len(self.textures)-1
        self.texture = self.textures[self.index]

    def draw(self):
        self.next()
        l,t = self.texture.texels[0]
        r,b = self.texture.texels[1]
        glBindTexture(GL_TEXTURE_2D, self.texture.pack.id)
        glBegin(GL_QUADS)
        w,h = self.texture.n_size
        glTexCoord2f(l,b);glVertex2f(0, 0)
        glTexCoord2f(l,t);glVertex2f(0, h)
        glTexCoord2f(r,t);glVertex2f(w, h)
        glTexCoord2f(r,b);glVertex2f(w, 0)
        glEnd()


class Tileset(Shape):
    def __init__(self, texture, (nw,nh), **kw):
        Shape.__init__(self, **kw)
        self.texture = texture
        self.nw = nw
        self.nh = nh
        texture_width = self.texture.texels[1][0] - self.texture.texels[0][0]
        texture_height = self.texture.texels[1][1] - self.texture.texels[0][1]

        self.tile_width = self.texture.n_size[0] / nw
        self.tile_height = self.texture.n_size[1] / nh

        self.xi = texture_width / nw
        self.yi = texture_height / nh

        print texture_height, self.xi, self.yi

        try:
            self.id = texture.pack.id
        except AttributeError:
            raise AttributeError, 'texture pack has no id attribute, (call texture,pack.build() first)'

    def get(self, (x,y)):
        class Tile(Shape):
            def __init__(_self):
                Shape.__init__(_self, centre_point = (-self.xi/2,self.yi/2))
                print _self.centre_point
            def draw(_self):
                xo = x * self.xi
                yo = y * self.yi
                l,b = self.texture.texels[0]
                l += xo
                b += yo
                t = b + self.yi
                r = l + self.xi

                glBindTexture(GL_TEXTURE_2D, self.id)

                glBegin(GL_QUADS)
                w,h = self.tile_width, self.tile_height
                glTexCoord2f(l,t);glVertex2f(0, 0)
                glTexCoord2f(l,b);glVertex2f(0, h)
                glTexCoord2f(r,b);glVertex2f(w, h)
                glTexCoord2f(r,t);glVertex2f(w, 0)
                glEnd()
        return Tile()

class Model(Shape):
    def __init__(self, filename, **kw):
        Shape.__init__(self,**kw)
        self.model = OBJFile.OBJFile(filename)
        self.rotation_x = 0.0
        self.rotation_y = 0.0
        self.rotation_z = 0.0

    def render(self):
        x,y = self.centre_point
        glPushMatrix()
        glTranslatef(x,y,0)
        self.draw()

        glPopMatrix()

    def draw(self):
        glBindTexture(GL_TEXTURE_2D, 0)
        glRotatef(self.rotation_x, 1, 0, 0)
        glRotatef(self.rotation_y, 0, 1, 0)
        glRotatef(self.rotation_z, 0, 0, 1)
        self.model.draw()

class Line(Shape):
    def __init__(self, points, **kw):
        self.connected=False
        self.points = points
        Shape.__init__(self, **kw)

    def draw(self):
        glBindTexture(GL_TEXTURE_2D, 0)
        if self.connected:
            glBegin(GL_LINE_STRIP)
        else:
            glBegin(GL_LINES)
        for point in self.points:
            glVertex2f(*point)
        glEnd()

class Polygon(Shape):
    def __init__(self, points, **kw):
        Shape.__init__(self, **kw)
        self.points = points

    def draw(self):
        glBindTexture(GL_TEXTURE_2D, 0)
        glBegin(GL_POLYGON)
        for point in self.points:
            glVertex2f(*point)
        glEnd()


