import time

class NanoThreadPool(object):
    """
    A cooperative thread scheduler.
    This class uses generator functions to implement cooperative threads.
    """
    __slots__ = ['threads','cleanups','real_threads']

    def __init__(self):
        self.threads = []
        self.cleanups = {}
        self.real_threads = []


    def __len__(self):
        """
        Returns a count of registered threads.
        """
        return len(self.threads)

    def register(self, threadfunc, *args, **kw):
        """
        Register a nanothread (threadfunc) with specified arguments.
        """
        thread = threadfunc(*args, **kw)
        self.threads.append(thread)
        return thread

    def register_thread(self, threadfunc, *args, **kw):
        from threading import Thread
        thread_delay = None
        if "thread_delay" in kw:
            thread_delay = kw['thread_delay']
            del kw['thread_delay']

        class NThread(Thread):
            def __init__(self):
                Thread.__init__(self)
            def run(_self):
                while True:
                    if _self._request_exit == True: return
                    if _self._delay: time.sleep(_self._delay)
                    try:
                        print _self.threadfunc.next()
                    except StopIteration:
                        return

        nt = NThread()
        nt.threadfunc = threadfunc(*args, **kw)
        nt._request_exit = False
        nt._delay = thread_delay
        btr = nt.start()

        self.real_threads.append(nt)
        return nt

    def kill_thread(self, thread):
        try:
            self.real_threads.remove(thread)
        except ValueError:
            pass
        thread._request_exit = True
        print 'set to kill', thread
        return self.exit(thread)

    def register_exit(self, thread, func, *args, **kw):
        self.cleanups[thread] = func,args,kw

    def terminate(self, thread):
        try:
            self.threads.remove(thread)
        except ValueError:
            pass

    def kill(self, thread):
        try:
            self.threads.remove(thread)
        except ValueError:
            pass
        return self.exit(thread)

    def exit(self, thread):
        try:
            func = self.cleanups[thread]
        except KeyError:
            pass
        else:
            return func[0](*func[1], **func[2])

    def defer(self, func, *args, **kw):
        """
        Register a function, which is to be called later, usually after the calling function has returned.
        """
        def deferred():
            func(*args, **kw)
            yield None

        return self.register(deferred)

    def defer_for(self, seconds, func, *args, **kw):
        """
        Register a function, which is to be called after a time period.
        """
        start = time.time()
        def deferred(s):
            time_time = time.time
            while (time_time() - s) <= seconds:
                yield None
            func(*args, **kw)

        return self.register(deferred, start)

    def loop(self):
        """
        Runs all threads. Exits when the last thread is finished.
        """
        while len(self.threads) > 0:
            self.poll()

    def poll(self):
        """
        Iterate once through all threads.
        """
        for thread  in (x for x in self.threads):
            try:
                thread.next()
            except StopIteration:
                self.kill(thread)


__TP = NanoThreadPool()
poll = __TP.poll
register = __TP.register
register_thread = __TP.register_thread
register_exit= __TP.register_exit
kill = __TP.kill
kill_thread = __TP.kill_thread
loop = __TP.loop
defer = __TP.defer
defer_for = __TP.defer_for
terminate = __TP.terminate

if __name__ == "__main__":
    pass
